# -*- coding: utf-8 -*-
from btsprice.exchanges import Exchanges
from btsprice.yahoo import Yahoo
from btsprice.sina import Sina
import time
import asyncio


class TaskExchanges(object):
    def __init__(self, data={}):
        self.period = 120
        self.exchanges = Exchanges()
        self.yahoo = Yahoo()
        self.sina = Sina()
        self.handler = None
        data_type = ["orderbook", "ticker", "rate"]
        for _type in data_type:
            if _type not in data:
                data[_type] = {}
        self.data = data

    def set_period(self, sec):
        self.period = sec

    @asyncio.coroutine
    def fetch_orderbook(self, name, quote, coro, *args):
        time_end = int(time.time())
        orderbook = self.data["orderbook"]
        while True:
            time_begin = time_end
            _orderbook = yield from coro(*args)
            time_end = int(time.time())
            if _orderbook:
                orderbook[name] = _orderbook
                orderbook[name]["quote"] = quote
                if "time" not in _orderbook:
                    orderbook[name]["time"] = time_end
                if self.handler:
                    self.handler("orderbook", name, orderbook[name])
            time_left = self.period - (time_end - time_begin)
            if time_left <= 1:
                time_left = 1
            time_end += time_left
            yield from asyncio.sleep(time_left)

    @asyncio.coroutine
    def fetch_ticker(self, name, quote, coro, *args):
        time_end = int(time.time())
        ticker = self.data["ticker"]
        while True:
            time_begin = time_end
            _ticker = yield from coro(*args)
            time_end = int(time.time())
            if _ticker:
                _ticker["quote"] = quote
                if "time" not in _ticker:
                    _ticker["time"] = time_end
                ticker[name] = _ticker
                if self.handler:
                    self.handler("ticker", name, _ticker)
            time_left = self.period - (time_end - time_begin)
            if time_left <= 1:
                time_left = 1
            time_end += time_left
            yield from asyncio.sleep(time_left)

    @asyncio.coroutine
    def fetch_yahoo_rate(self):
        time_end = int(time.time())
        rate = self.data["rate"]
        while True:
            time_begin = time_end
            _rate = yield from self.yahoo.fetch_price()
            time_end = int(time.time())
            if _rate:
                _rate["time"] = time_end
                rate["yahoo"] = _rate
                if self.handler:
                    self.handler("rate", "yahoo", _rate)
            time_left = self.period - (time_end - time_begin)
            if time_left <= 1:
                time_left = 1
            time_end += time_left
            yield from asyncio.sleep(time_left)

    @asyncio.coroutine
    def fetch_sina_rate(self):
        time_end = int(time.time())
        rate = self.data["rate"]
        while True:
            time_begin = time_end
            _rate = yield from self.sina.fetch_price()
            time_end = int(time.time())
            if _rate:
                _rate["time"] = time_end
                rate["Sina"] = _rate
                if self.handler:
                    self.handler("rate", "Sina", _rate)
            time_left = self.period - (time_end - time_begin)
            if time_left <= 1:
                time_left = 1
            time_end += time_left
            yield from asyncio.sleep(time_left)

    def run_tasks_ticker(self, loop):
        return [
            loop.create_task(self.fetch_ticker(
                "btc38", "CNY",
                self.exchanges.ticker_btc38, "cny", "btc")),
            loop.create_task(self.fetch_ticker(
                "poloniex", "USD",
                self.exchanges.ticker_poloniex, "USDT", "BTC")),
            loop.create_task(self.fetch_ticker(
                "btce", "USD",
                self.exchanges.ticker_btce, "usd", "btc")),
            loop.create_task(self.fetch_ticker(
                "bitstamp", "USD",
                self.exchanges.ticker_bitstamp, "usd", "btc")),
            loop.create_task(self.fetch_ticker(
                "gdax", "USD",
                self.exchanges.ticker_gdax, "usd", "btc")),
            loop.create_task(self.fetch_ticker(
                "btcchina", "CNY",
                self.exchanges.ticker_btcchina, "cny", "btc")),
            loop.create_task(self.fetch_ticker(
                "huobi", "CNY",
                self.exchanges.ticker_huobi, "btc")),
            loop.create_task(self.fetch_ticker(
                "okcoin_cn", "CNY",
                self.exchanges.ticker_okcoin_cn, "cny", "btc")),
            loop.create_task(self.fetch_ticker(
                 "okcoin_com", "USD",
                 self.exchanges.ticker_okcoin_com, "usd", "btc")),
            ]

    def run_tasks_orderbook(self, loop):
        return [
            loop.create_task(self.fetch_orderbook(
                "btsbots_cny", "CNY",
                self.exchanges.orderbook_btsbots, "CNY", "BTS")),
            loop.create_task(self.fetch_orderbook(
                "btsbots_usd", "USD",
                self.exchanges.orderbook_btsbots, "USD", "BTS")),
            loop.create_task(self.fetch_orderbook(
                "btsbots_open.btc", "BTC",
                self.exchanges.orderbook_btsbots, "OPEN.BTC", "BTS")),
            loop.create_task(self.fetch_orderbook(
                "btc38_cny", "CNY",
                self.exchanges.orderbook_btc38, "cny", "bts")),
            loop.create_task(self.fetch_orderbook(
                "chbtc_cny", "CNY",
                self.exchanges.orderbook_chbtc, "cny", "bts")),
            loop.create_task(self.fetch_orderbook(
                "jubi_cny", "CNY",
                self.exchanges.orderbook_jubi, "cny", "bts")),
            loop.create_task(self.fetch_orderbook(
                "19800_cny", "CNY",
                self.exchanges.orderbook_19800, "cny", "bts")),
            loop.create_task(self.fetch_orderbook(
                "poloniex_btc", "BTC",
                self.exchanges.orderbook_poloniex, "btc", "bts")),
            loop.create_task(self.fetch_orderbook(
                "yunbi_cny", "CNY",
                self.exchanges.orderbook_yunbi, "cny", "bts"))
            ]

    def run_tasks(self, loop):
        return [
                loop.create_task(self.fetch_yahoo_rate()),
                loop.create_task(self.fetch_sina_rate())
                ] + \
            self.run_tasks_orderbook(loop) + \
            self.run_tasks_ticker(loop)


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    task_exchanges = TaskExchanges()
    task_exchanges.set_period(20)
    tasks = task_exchanges.run_tasks(loop)

    @asyncio.coroutine
    def task_display():
        my_data = task_exchanges.data
        while True:
            for _type in my_data:
                for _name in my_data[_type]:
                    if "done" not in my_data[_type][_name]:
                        print("got %s: %s" % (_type, _name))
                        my_data[_type][_name]["done"] = None
            yield from asyncio.sleep(1)
    tasks += [loop.create_task(task_display())]
    loop.run_until_complete(asyncio.wait(tasks))
    loop.run_forever()
    loop.close()



# -*- coding: utf-8 -*-
import time
import copy
from math import fabs
from btsprice.misc import get_median
# from pprint import pprint


class BTSPriceAfterMatch(object):

    def __init__(self, data):
        self.data = data
        self.timeout = 600  # order book can't use after 300 seconds
        self.order_types = ["bids", "asks"]
        self.orderbook = {}
        self.global_orderbook = {"bids": [], "asks": []}
        self.market_weight = {"poloniex_btc": 1, "btc38_cny": 1}
        self.rate_cny = {}
        self.callback = None

    def set_weight(self, _market_weight):
        self.market_weight = _market_weight

    def set_timeout(self, timeout):
        self.timeout = timeout

    def change_order_with_rate(self, _orderbook, rate):
        for order_type in self.order_types:
            for order in _orderbook[order_type]:
                order[0] = order[0] * rate

    def remove_timeout(self, data):
        for name in list(data.keys()):
            if self.timestamp - data[name]["time"] >= self.timeout:
                del data[name]

    def test_valid(self):
        valid_price_queue = []
        valid_price_dict = {}
        for market in self.market_weight:
            if market not in self.orderbook:
                continue
            if len(self.orderbook[market]["bids"]) == 0 or \
                    len(self.orderbook[market]["asks"]) == 0:
                    continue
            _price = (
                self.orderbook[market]["bids"][0][0] + self.
                orderbook[market]["asks"][0][0])/2
            if self.market_weight[market]:
                valid_price_queue.append(_price)
            valid_price_dict[market] = _price
        valid_price = get_median(valid_price_queue)
        for market in list(self.orderbook.keys()):
            if market not in valid_price_dict:
                del self.orderbook[market]
                continue
            change = fabs((valid_price_dict[market] - valid_price)/valid_price)
            # if offset more than 20%, this market is invalid
            if change > 0.2:
                del self.orderbook[market]
        # all market is invalid if we got valid market less than 2
        if len(self.orderbook) < 2:
            return False
        else:
            return True

    def compute_rate_cny(self):
        btc_ticker = self.data["ticker"].copy()
        self.remove_timeout(btc_ticker)
        price_btc_queue = {"CNY": [], "USD": []}
        price_btc = {}
        for name in btc_ticker:
            quote = btc_ticker[name]["quote"]
            price_btc_queue[quote].append(btc_ticker[name]["last"])
        for asset in price_btc_queue:
            if len(price_btc_queue[asset]) == 0:
                return
            price_btc[asset] = get_median(price_btc_queue[asset])
            # print(price_btc[asset], price_btc_queue[asset])
        rate_cny = {"CNY": 1.0}
        rate_cny["BTC"] = price_btc["CNY"]
        rate_cny["USD"] = price_btc["CNY"] / price_btc["USD"]

        rate = self.data["rate"]
        if len(rate) == 0:
            return
        # rate_yahoo = rate["yahoo"]
        # rate_cny["USD"] = 1/rate_yahoo["USD"]["CNY"]

        rate_cny["BTC"] = 0.0
        for asset in price_btc:
            rate_cny["BTC"] += price_btc[asset] * rate_cny[asset]
        rate_cny["BTC"] /= len(price_btc)

        rate_source = {}
        for quote in ["CNY", "USD"]:
            for source in list(rate):
                for base in rate[source][quote]:
                    if base in rate_cny:
                        continue
                    if base not in rate_source:
                        rate_source[base] = []
                    _rate = rate[source][quote][base] * rate_cny[quote]
                    rate_source[base].append(_rate)
            for asset in rate_source:
                asset_rate = sum(rate_source[asset])/float(len(rate_source[asset]))
                for _rate in list(rate_source[asset]):
                    if fabs((_rate - asset_rate)/asset_rate) > 0.1:
                        asset_rate = None
                        break
                if asset_rate:
                    rate_cny[asset] = asset_rate

        rate_cny["TCNY"] = rate_cny["CNY"]
        rate_cny["TUSD"] = rate_cny["USD"]
        self.rate_cny = rate_cny

    def update_orderbook(self):
        if not self.rate_cny:
            return
        self.global_orderbook = {"bids": [], "asks": []}
        self.orderbook = copy.deepcopy(self.data["orderbook"])
        self.remove_timeout(self.orderbook)
        for market in self.orderbook:
            change_rate = self.rate_cny[self.orderbook[market]["quote"]]
            self.change_order_with_rate(self.orderbook[market], change_rate)
        if not self.test_valid():
            return
        if self.callback:
            self.callback(self.orderbook)
        for market in self.orderbook:
            for order_type in self.order_types:
                self.global_orderbook[order_type].extend(
                    self.orderbook[market][order_type])
        self.global_orderbook["bids"] = sorted(
            self.global_orderbook["bids"], reverse=True)
        self.global_orderbook["asks"] = sorted(self.global_orderbook["asks"])

    def get_spread_orderbook(self, spread=0.01):
        order_bids = []
        order_asks = []
        for order in self.global_orderbook["bids"]:
            order_bids.append([order[0]*(1 + spread), order[1]])
        for order in self.global_orderbook["asks"]:
            order_asks.append([order[0]/(1 + spread), order[1]])
        return order_bids, order_asks

    def get_price_list(self, order_bids, order_asks):
        price_list = []
        for order in order_bids:
            if order[0] < order_asks[0][0]:
                break
            if len(price_list) == 0 or order[0] != price_list[-1]:
                price_list.append(order[0])
        for order in order_asks:
            if order[0] < order_bids[0][0]:
                break
            if len(price_list) == 0 or order[0] != price_list[-1]:
                price_list.append(order[0])
        price_list = sorted(price_list)
        return price_list

    def get_match_result(self, order_bids, order_asks, price_list):
        bid_volume = ask_volume = 0
        median_price = get_median(price_list)
        for order in order_bids:
            if order[0] < median_price:
                break
            bid_volume += order[1]
        for order in order_asks:
            if order[0] > median_price:
                break
            ask_volume += order[1]
        return ([bid_volume, ask_volume, median_price])

    def compute_price(self, spread=0.01):
        self.timestamp = int(time.time())
        self.compute_rate_cny()
        self.update_orderbook()
        if len(self.global_orderbook["bids"]) == 0 or \
                len(self.global_orderbook["asks"]) == 0:
            return [0.0, 0.0, None]
        order_bids, order_asks = self.get_spread_orderbook(spread)
        price_list = self.get_price_list(order_bids, order_asks)
        if len(price_list) < 1:
            return [0.0, 0.0, (order_bids[0][0] + order_asks[0][0]) / 2]
        match_result = []
        while True:
            bid_volume, ask_volume, median_price = self.get_match_result(
                order_bids, order_asks, price_list)
            # we need to find a price, which can match the max volume
            match_result.append([min(bid_volume, ask_volume),
                                 -(bid_volume+ask_volume), median_price])
            if len(price_list) <= 1:
                break
            if(bid_volume <= ask_volume):
                price_list = price_list[:int(len(price_list) / 2)]
            else:
                price_list = price_list[int(len(price_list) / 2):]

        match_result = sorted(match_result, reverse=True)
        # pprint(match_result)
        return match_result[0]

    def get_valid_depth(self, price, spread=0.01):
        valid_depth = {}
        bid_price = price / (1+spread)
        ask_price = price * (1+spread)
        for market in self.orderbook:
            quote = self.orderbook[market]["quote"]
            valid_depth[market] = {
                "bid_price": bid_price / self.rate_cny[quote], "bid_volume": 0,
                "ask_price": ask_price / self.rate_cny[quote], "ask_volume": 0}
            for order in sorted(self.orderbook[market]["bids"], reverse=True):
                if order[0] < bid_price:
                    break
                # valid_depth[market]["bid_price"] = \
                #     order[0] * self.rate_cny[quote]
                valid_depth[market]["bid_volume"] += order[1]
            for order in sorted(self.orderbook[market]["asks"]):
                if order[0] > ask_price:
                    break
                # valid_depth[market]["ask_price"] = \
                #     order[0] * self.rate_cny[quote]
                valid_depth[market]["ask_volume"] += order[1]
        return valid_depth

if __name__ == "__main__":
    import asyncio
    from btsprice.task_exchanges import TaskExchanges
    exchange_data = {}
    task_exchanges = TaskExchanges(exchange_data)
    task_exchanges.set_period(20)
    loop = asyncio.get_event_loop()
    task_exchanges.run_tasks(loop)

    bts_price = BTSPriceAfterMatch(exchange_data)

    @asyncio.coroutine
    def task_compute_price():
        yield from asyncio.sleep(1)
        while True:
            volume, volume_sum, real_price = bts_price.compute_price(
                spread=0.01)
            if real_price:
                valid_depth = bts_price.get_valid_depth(
                    price=real_price,
                    spread=0.01)
                print(
                    "price:%.5f CNY/BTS, volume:%.3f" % (real_price, volume))
                print("efficent depth : %s" % valid_depth)
            else:
                print("can't get valid market order")
            yield from asyncio.sleep(20)
    loop.create_task(task_compute_price())
    loop.run_forever()
    loop.close()



# -*- coding: utf-8 -*-
"""Project btsprice

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'btsprice'
project = "btsprice"
project_no_spaces = project.replace(' ', '')
version = '0.2.43'
description = 'get price for BitShares'
authors = ['Alt']
authors_string = ', '.join(authors)
emails = ['pch957@163.com']
license = 'MIT'
copyright = '2015 ' + authors_string
url = 'https://github.com/pch957/btsprice'



# -*- coding: utf-8 -*-
import json
import asyncio
import aiohttp
import datetime


class Exchanges():
    def __init__(self):
        header = {
            'User-Agent': 'curl/7.35.0',
            'Accept': '*/*'}

        self.session = aiohttp.ClientSession(headers=header)
        self.order_types = ["bids", "asks"]

    @asyncio.coroutine
    def orderbook_btc38(self, quote="cny", base="bts"):
        try:
            url = "http://api.btc38.com/v1/depth.php"
            params = {'c': base, 'mk_type': quote}
            response = yield from asyncio.wait_for(self.session.get(
                url, params=params), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except:
            print("Error fetching book from btc38!")

    @asyncio.coroutine
    def orderbook_bter(self, quote="cny", base="bts"):
        try:
            url = "http://data.bter.com/api/1/depth/%s_%s" % (base, quote)
            response = yield from asyncio.wait_for(self.session.get(
                url), 120)
            result = yield from response.json()
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except:
            print("Error fetching book from bter!")

    @asyncio.coroutine
    def orderbook_yunbi(self, quote="cny", base="bts"):
        try:
            url = "https://yunbi.com/api/v2/depth.json"
            params = {'market': base+quote, 'limit': 20}
            response = yield from asyncio.wait_for(self.session.get(
                url, params=params), 120)
            result = yield from response.json()
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            time = int(result["timestamp"])
            return {
                "bids": order_book_bid, "asks": order_book_ask, "time": time}
        except:
            print("Error fetching book from yunbi!")

    @asyncio.coroutine
    def orderbook_btsbots(self, quote="CNY", base="BTS"):
        try:
            url = "https://btsbots.com/api/order?max_results=30&where="
            # url = "http://localhost:5000/api/order?max_results=30&where="
            params = "a_s==%s;a_b==%s" % (base, quote)
            response = yield from asyncio.wait_for(self.session.get(
                url+params), 120)
            result = yield from response.json()
            order_book_ask = []
            for _o in result["_items"]:
                order_book_ask.append([_o['p'], _o['b_s']])
            params = "a_s==%s;a_b==%s" % (quote, base)
            response = yield from asyncio.wait_for(self.session.get(
                url+params), 120)
            result = yield from response.json()
            order_book_bid = []
            for _o in result["_items"]:
                order_book_bid.append([1/_o['p'], _o['b_b']])
            return {
                "bids": order_book_bid, "asks": order_book_ask}
        except:
            print("Error fetching book from btsbots!")

    @asyncio.coroutine
    def orderbook_poloniex(self, quote="btc", base="bts"):
        try:
            quote = quote.upper()
            base = base.upper()
            url = "http://poloniex.com/public"
            params = {
                "command": "returnOrderBook",
                "currencyPair": "%s_%s" % (quote, base),
                "depth": 150
                }

            response = yield from asyncio.wait_for(self.session.get(
                url, params=params), 120)
            result = yield from response.json()
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except Exception as e:
            print("Error fetching book from poloniex!")

    @asyncio.coroutine
    def orderbook_chbtc(self, quote="cny", base="bts"):
        try:
            quote = quote.lower()
            base = base.lower()
            url = "http://api.chbtc.com/data/v1/depth"
            params = {
                "currency": "%s_%s" % (base, quote),
                "size": 150
                }
            response = yield from asyncio.wait_for(self.session.get(
                url, params=params), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except Exception as e:
            print("Error fetching book from chbtc!")
            print(e)

    @asyncio.coroutine
    def orderbook_jubi(self, quote="cny", base="bts"):
        try:
            quote = quote.lower()
            base = base.lower()
            url = "https://www.jubi.com/api/v1/depth"
            params = {
                "coin": "%s" % (base)
                }
            response = yield from asyncio.wait_for(self.session.get(
                url, params=params), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            for order_type in self.order_types:
                for order in result[order_type]:
                    order[0] = float(order[0])
                    order[1] = float(order[1])
            order_book_ask = sorted(result["asks"])
            order_book_bid = sorted(result["bids"], reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except Exception as e:
            print("Error fetching book from jubi!")
            print(e)

    @asyncio.coroutine
    def orderbook_19800(self, quote="cny", base="bts"):
        try:
            quote = quote.lower()
            base = base.lower()
            url = "https://www.19800.com/api/v1/depth"
            params = {
                "market": "%s_%s" % (quote, base)
                }
            response = yield from asyncio.wait_for(self.session.get(
                url, params=params), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            result = result['data']
            order_book_ask = []
            order_book_bid = []
            for order in result['bids']:
                order_book_bid.append([float(order['Price']), float(order['Volume'])])
            for order in result['asks']:
                order_book_ask.append([float(order['Price']), float(order['Volume'])])
            order_book_ask = sorted(order_book_ask)
            order_book_bid = sorted(order_book_bid, reverse=True)
            return {"bids": order_book_bid, "asks": order_book_ask}
        except Exception as e:
            print("Error fetching book from 19800!")
            print(e)

    @asyncio.coroutine
    def ticker_btc38(self, quote="cny", base="bts"):
        try:
            url = "http://api.btc38.com/v1/ticker.php?c=%s&mk_type=%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result['ticker']["last"]
            _ticker["vol"] = result['ticker']["vol"]
            _ticker["buy"] = result['ticker']["buy"]
            _ticker["sell"] = result['ticker']["sell"]
            _ticker["low"] = result['ticker']["low"]
            _ticker["high"] = result['ticker']["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["name"] = "btc38"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from btc38!")
            print(e)

    @asyncio.coroutine
    def ticker_poloniex(self, quote="USDT", base="BTC"):
        try:

            url = "https://poloniex.com/public?command=returnTicker"
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(
                response.decode("utf-8-sig"))["%s_%s" % (quote, base)]
            _ticker = {}
            _ticker["last"] = result["last"]
            _ticker["vol"] = result["baseVolume"]
            _ticker["buy"] = result["highestBid"]
            _ticker["sell"] = result["lowestAsk"]
            _ticker["low"] = result["low24hr"]
            _ticker["high"] = result["high24hr"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["name"] = "poloniex"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from poloniex!")
            print(e)

    @asyncio.coroutine
    def ticker_btcchina(self, quote="cny", base="btc"):
        try:
            url = "https://data.btcchina.com/data/ticker?market=%s%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result['ticker']["last"]
            _ticker["vol"] = result['ticker']["vol"]
            _ticker["buy"] = result['ticker']["buy"]
            _ticker["sell"] = result['ticker']["sell"]
            _ticker["low"] = result['ticker']["low"]
            _ticker["high"] = result['ticker']["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['ticker']["date"])
            _ticker["name"] = "btcchina"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from btcchina!")
            print(e)

    @asyncio.coroutine
    def ticker_huobi(self, base="btc"):
        try:
            url = "http://api.huobi.com/staticmarket/ticker_%s_json.js" % base
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result['ticker']["last"]
            _ticker["vol"] = result['ticker']["vol"]
            _ticker["buy"] = result['ticker']["buy"]
            _ticker["sell"] = result['ticker']["sell"]
            _ticker["low"] = result['ticker']["low"]
            _ticker["high"] = result['ticker']["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['time'])
            _ticker["name"] = "huobi"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from huobi!")
            print(e)

    @asyncio.coroutine
    def ticker_okcoin_cn(self, quote="cny", base="btc"):
        try:
            url = "https://www.okcoin.cn/api/ticker.do?symbol=%s_%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result['ticker']["last"]
            _ticker["vol"] = result['ticker']["vol"]
            _ticker["buy"] = result['ticker']["buy"]
            _ticker["sell"] = result['ticker']["sell"]
            _ticker["low"] = result['ticker']["low"]
            _ticker["high"] = result['ticker']["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['date'])
            _ticker["name"] = "okcoin.cn"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from okcoin cn!")
            print(e)

    @asyncio.coroutine
    def ticker_okcoin_com(self, quote="usd", base="btc"):
        try:
            url = "https://www.okcoin.com/api/v1/ticker.do?symbol=%s_%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result['ticker']["last"]
            _ticker["vol"] = result['ticker']["vol"]
            _ticker["buy"] = result['ticker']["buy"]
            _ticker["sell"] = result['ticker']["sell"]
            _ticker["low"] = result['ticker']["low"]
            _ticker["high"] = result['ticker']["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['date'])
            _ticker['name'] = 'okcoin.com'
            return _ticker
        except Exception as e:
            print("Error fetching ticker from okcoin com!")
            print(e)

    @asyncio.coroutine
    def ticker_gdax(self, quote="usd", base="btc"):
        try:
            url = "https://api.gdax.com/products/%s-%s/ticker" % (
                base.upper(), quote.upper())
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result["price"]
            _ticker["vol"] = result["volume"]
            _ticker["buy"] = result["bid"]
            _ticker["sell"] = result["ask"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["low"] = None
            _ticker["high"] = None
            _ticker["time"] = int(
                datetime.datetime.strptime(
                    result["time"][:19]+"+0000", "%Y-%m-%dT%H:%M:%S%z").timestamp())
            _ticker["name"] = "gdax"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from gdax.com!")
            print(e)

    @asyncio.coroutine
    def ticker_bitstamp(self, quote="usd", base="btc"):
        try:
            url = "https://www.bitstamp.net/api/v2/ticker/%s%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            _ticker = {}
            _ticker["last"] = result["last"]
            _ticker["vol"] = result["volume"]
            _ticker["buy"] = result["bid"]
            _ticker["sell"] = result["ask"]
            _ticker["low"] = result["low"]
            _ticker["high"] = result["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['timestamp'])
            _ticker["name"] = "bitstamp"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from bitstamp.net!")
            print(e)

    @asyncio.coroutine
    def ticker_btce(self, quote="usd", base="btc"):
        try:
            url = "https://btc-e.com/api/3/ticker/%s_%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            result = result["%s_%s" % (base, quote)]
            _ticker = {}
            _ticker["last"] = result["last"]
            _ticker["vol"] = result["vol_cur"]
            _ticker["buy"] = result["buy"]
            _ticker["sell"] = result["sell"]
            _ticker["low"] = result["low"]
            _ticker["high"] = result["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['updated'])
            _ticker["name"] = "btce"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from btc-e.com!")
            print(e)

    @asyncio.coroutine
    def ticker_bitflyer(self, quote="usd", base="btc"):
        try:
            url = "https://api.bitflyer.jp/v1/%s_%s" % (
                base, quote)
            response = yield from asyncio.wait_for(self.session.get(url), 120)
            response = yield from response.read()
            result = json.loads(response.decode("utf-8-sig"))
            result = result["%s_%s" % (base, quote)]
            _ticker = {}
            _ticker["last"] = result["last"]
            _ticker["vol"] = result["vol_cur"]
            _ticker["buy"] = result["buy"]
            _ticker["sell"] = result["sell"]
            _ticker["low"] = result["low"]
            _ticker["high"] = result["high"]
            for key in _ticker:
                _ticker[key] = float(_ticker[key])
            _ticker["time"] = int(result['updated'])
            _ticker["name"] = "btce"
            return _ticker
        except Exception as e:
            print("Error fetching ticker from btc-e.com!")
            print(e)

if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    exchanges = Exchanges()

    @asyncio.coroutine
    def run_task(coro, *args):
        while True:
            result = yield from coro(*args)
            print(result)
            yield from asyncio.sleep(120)

    tasks = [
        # loop.create_task(run_task(exchanges.orderbook_btsbots)),
        # loop.create_task(run_task(exchanges.orderbook_btsbots, "OPEN.BTC", "BTS")),
        # loop.create_task(run_task(exchanges.orderbook_btc38)),
        loop.create_task(run_task(exchanges.orderbook_chbtc)),
        loop.create_task(run_task(exchanges.orderbook_19800))
        # loop.create_task(run_task(exchanges.orderbook_btc38, "cny", "btc")),
        # loop.create_task(run_task(exchanges.orderbook_yunbi)),
        # loop.create_task(run_task(exchanges.orderbook_poloniex)),
        # loop.create_task(run_task(exchanges.ticker_btc38)),
        # loop.create_task(run_task(exchanges.ticker_gdax)),
        # loop.create_task(run_task(exchanges.ticker_btcchina)),
        # loop.create_task(run_task(exchanges.ticker_huobi)),
        # loop.create_task(run_task(exchanges.ticker_okcoin_cn)),
        # loop.create_task(run_task(exchanges.ticker_okcoin_com))
        ]
    loop.run_until_complete(asyncio.wait(tasks))
    loop.run_forever()
